#!/bin/bash -l
set -euf -o pipefail
shopt -s inherit_errexit

TXT_RED="\e[31m"
TXT_BLUE="\e[94m"
TXT_GREEN="\e[32m"
TXT_CLEAR="\e[0m"
TXT_BOLD="\e[1m"

hash awk
hash chown
hash diff
hash getent
hash id
hash runuser
hash sed
hash ssh-keygen

function error {
    : "${RV:=${2:-$BUILD_FAILURE_EXIT_CODE}}"
    echo -e "${TXT_RED}${TXT_BOLD}$1${TXT_CLEAR}" >&2
    return "$RV"
}

## User authentication

[ -z "${CUSTOM_ENV_AUTH_USER:+x}" ] && error "AUTH_USER CI/CD variable has not been set."
[ -z "${CUSTOM_ENV_AUTH_KEY:+x}" ] && error "AUTH_KEY secret CI/CD variable has not been set."

AUTH_USER=$CUSTOM_ENV_AUTH_USER

## Check if the user exists and the validity of its ID
id -u "$AUTH_USER" >/dev/null 2>&1 || error "User $AUTH_USER does not exist"
(( $(id -u "$AUTH_USER") >= 1000 )) || error "User $AUTH_USER ID within system reserved range."

## Use a key pair to authenticate the user (private key has to be set as a GitLab CI/CD variable)
AUTH_KEY=$CUSTOM_ENV_AUTH_KEY
AUTH_PUB="/etc/gitlab-runner/auth/${CUSTOM_ENV_CI_SERVER_HOST}/${CUSTOM_ENV_CI_PROJECT_NAMESPACE}/${CUSTOM_ENV_CI_PROJECT_NAME}/authorized_keys"

if [[ ! -f "$AUTH_PUB" ]]; then
    error "Authentication for user $AUTH_USER not configured for this project."
fi

(
while read -r PUB
do
    diff <(ssh-keygen -y -e -f /dev/stdin <<< "$AUTH_KEY") <(ssh-keygen -y -e -f /dev/stdin <<< "$PUB") && exit 0
done < "$AUTH_PUB"
exit 1
) || error "Authentication for user $AUTH_USER failed."

### Env cleanup

AUTH_USER_WORK=$(runuser "$AUTH_USER" --login --command "echo \$WORK")

[ -z "${AUTH_USER_WORK:+x}" ] && error "Could not determine $AUTH_USER's \$WORK directory."

BASE_DIR=$AUTH_USER_WORK/gitlab-runner/builds/${CUSTOM_ENV_CI_CONCURRENT_PROJECT_ID:?}/${CUSTOM_ENV_CI_PROJECT_NAMESPACE:?}/${CUSTOM_ENV_CI_PROJECT_NAME:?}

runuser "$AUTH_USER" --command "rm -rf $BASE_DIR"
runuser "$AUTH_USER" --command "rm -rf $BASE_DIR.tmp"
